@@ -3,15 +3,17 @@ package Module::Build::FFI;
 use strict;
 use warnings;
 use base qw( Module::Build );
+use Config;
 use Carp qw( croak );
 use File::Spec;
 use File::Temp qw( tempdir );
 use File::Copy qw( copy cp );
 use File::Basename qw( basename );
 use File::ShareDir qw( dist_dir );
+use Text::ParseWords qw(shellwords);
 
 # ABSTRACT: Build Perl extensions if C with FFI.
-our $VERSION = '0.04'; # VERSION
+our $VERSION = '0.05'; # VERSION
 
 
 sub new
@@ -19,7 +21,6 @@ sub new
   my($class, %args) = @_;
   
   $args{c_source} ||= 'ffi';
-  $args{requires}{'Alien::o2dll'} = '0' if $^O eq 'MSWin32';
   
   my $dir = eval { File::Spec->catdir(dist_dir('FFI-Util'), 'include') };
   unless($@)
@@ -54,26 +55,21 @@ sub link_c
 
   if($^O eq 'MSWin32')
   {
-    # TODO: rm dep on Alien::o2dll
-    require Alien::o2dll;
-    my $dir = tempdir( CLEANUP => 1 );
-    
-    foreach my $object (@$objects)
+    my @cmd;
+    if($Config{cc} !~ /cl(\.exe)?$/) # MSWin32 + GCC
     {
-      copy($object, File::Spec->catfile($dir, basename $object)) || die "unable to copy $!";
+      my $lddlflags = $Config{lddlflags};
+      $lddlflags =~ s{\\}{/}g;
+      @cmd = ($Config{cc}, shellwords($lddlflags), -o => $spec->{lib_file}, "-Wl,--export-all-symbols", @$objects);
     }
-    
-    my $save = Win32::GetCwd();
-    chdir $dir;
-    Alien::o2dll::o2dll( -o => basename($spec->{lib_file}), map { basename $_ } @$objects );
-    
-    opendir my $dh, '.';
-    my($tmp_dll_name) = grep /\.dll$/, readdir $dh;
-    closedir  $dh;
-    
-    chdir $save;
-    
-    cp(File::Spec->catfile($dir, $tmp_dll_name), $spec->{lib_file}) || die "unable to copy $!";
+    else                             # MSWin32 + Visual C++
+    {
+      @cmd = ($Config{cc}, @$objects, "/link", "/dll", "/out:" . $spec->{lib_file});
+    }
+
+    print "@cmd\n";
+    system @cmd;
+    die "error in link" if $?;
   }
   else
   {
@@ -144,7 +140,7 @@ Module::Build::FFI - Build Perl extensions if C with FFI.
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 SYNOPSIS
 
